<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

namespace Vvveb\Plugins\FlatRateShipping;

use function Vvveb\__;
use function Vvveb\getLanguageId;
use function Vvveb\getMultiSettingContent;
use function Vvveb\getSetting;
use Vvveb\System\Core\View;
use Vvveb\System\ShippingMethod;

if (! defined('V_VERSION')) {
	die('Invalid request!');
}

class Shipping extends ShippingMethod {
	private $namespace = 'flat-rate-shipping';

	private $method_data = [
		'name'                    => 'flat-rate-shipping',
		'title'                   => 'Flat rate',
		'description'             => 'Fixed shipping rate',
		'region_id'               => 0,
		'cost'	                   => 0,
		'tax_type_id'             => 0,
		'region_group_id'         => 0,
		'free_shipping'           => 0,
		'free_shipping_message'   => 'Add more products worth at least %s to get free shipping!',
	];

	private function getCost() {
		$this->method_data = &$this->method_data;

		$this->method_data['text'] = '';
		$subtotal                  = $this->cart->getSubtotal();

		if ($this->method_data['free_shipping'] && ($subtotal > $this->method_data['free_shipping'])) {
			$this->method_data['cost']  = 0;
			$this->method_data['text']  = __('Free shipping');
		}
		{
			$this->method_data['difference'] = max($this->method_data['free_shipping'] - $subtotal, 0);
		}
	}

	public function getMethod() {
		$this->method_data = getSetting($this->namespace, null, []) + $this->method_data;

		$this->getCost();

		$region_id = 0;

		$language_id = getLanguageId();
		$lang        = getMultiSettingContent(SITE_ID, $this->namespace, [], $language_id) ?? [];

		foreach ($lang as $meta) {
			$desc[$meta['language_id']][$meta['key']] = $meta['value'];
		}

		$this->method_data['description'] = $desc[$language_id]['desc']['message'] ?? $this->method_data['description'];
		$this->method_data['title']       = $desc[$language_id]['desc']['name'] ?? $this->method_data['title'];

		if ($region_id == 0 || $region_id == $this->method_data['region_id']) {
			return $this->method_data;
		}
	}

	public function init() {
		//remove previous total and tax
		$this->cart->removeTotal($this->namespace);
		$this->cart->removeTax($this->namespace);
	}

	public function setMethod() {
		$view = View ::getInstance();

		if ($this->method_data['free_shipping_message'] && $this->method_data['difference']) {
			$view->info['flat-shipping-message'] = sprintf($this->method_data['free_shipping_message'], $this->method_data['difference']);
		}

		if ($this->method_data['tax_type_id']) {
			$this->cart->addTax($this->namespace, $this->method_data['cost'], $this->method_data['tax_type_id']);
		}
		$this->cart->addTotal($this->namespace, $this->method_data['title'] , $this->method_data['cost'], $this->method_data['text']);
	}
}
